//============================================================================//
//   Perceptron Simulator                                                     //
//                                                                            //
//   Chad Rempp    crempp@prons.net                                           //
//   July 1 2002                                                              //
//                                                                            //
//============================================================================//

#include <iostream.h>
#include <stdlib.h>
#include <vector.h>

void screen(void);
void edit_input(void);
void edit_desired_output(void);
void edit_weights(void);
void ptrain(void);
void ptest(void);
float act_val(void);
bool fire(void);

typedef float * data_t;

struct perceptron
{
      int bias;
      int n_inputs,
          n_outputs,
          iterations;
      float learningrate;
      vector <int> v_inputs;
      vector <int> v_outputs;
      vector <float> v_weights;
      vector <int> v_desiredoutputs;
};

perceptron p;

int main()
{
      char selection;

      while (selection != 'q')
      {
          screen();
          cout << "|> "; cin >> selection;
          switch (selection)
          {
              case 'j':
                  cout << "Enter the number of iterations : ";
                  cin >> p.iterations;
                  break;
              case 'r':
                  cout << "Enter the learning rate : ";
                  cin >> p.learningrate;
                  break;
              case 'i':
                  cout << "Enter the number of inputs : ";
                  if (!p.v_inputs.empty()) {p.v_inputs.clear(); p.v_weights.clear();}
                  cin >> p.n_inputs;
                  for (int i = 0; i < p.n_inputs; i++)
                      {p.v_inputs.push_back(0); p.v_weights.push_back(0);}
                  break;
              case 'o':
                  cout << "Enter the number of outputs : "; cin >> p.n_outputs;
                  if (!p.v_outputs.empty()) p.v_outputs.clear(); p.v_desiredoutputs.clear();
                  for (int i = 0; i < p.n_outputs; i++)
                      p.v_outputs.push_back(0);
                  for (int i = 0; i < p.n_outputs; i++)
                      p.v_desiredoutputs.push_back(0);
                  break;
              case 'b':
                  cout << "Enter the bias value : "; cin >> p.bias;
                  break;
              case 'v':
                  edit_input();
                  break;
              case 'w':
                  edit_weights();
                  break;
              case 'd':
                  edit_desired_output();
                  break;
              case 't':
                  ptrain();
                  break;
              case 'g':
                  ptest();
                  break;
              case 'q':
                  break;
              default:
                  cout << "Invalid selection" << endl;
          }
      }
      return 0;
}

void screen(void)
{
      cout << endl;
      cout << "+-----------------------------------------------------------------------------+" << endl;
      cout << "| Binary Perceptron Simulator                                                 |" << endl;
      cout << "| version 0.1    by Chad Rempp    July 1st 2002                               |" << endl;
      cout << "+-----------------------------------------------------------------------------+" << endl;
      cout << "                  |Change | Current Value" << endl;
      cout << "-------------------------------------------------------------------------------" << endl;
      cout << "Iterations        |   j   | " << p.iterations << endl;
      cout << "Learning Rate     |   r   | " << p.learningrate << endl;
      cout << "Number of Inputs  |   i   | " << p.n_inputs << endl;
      cout << "Number of Outputs |   o   | " << p.n_outputs << endl;
      cout << "Bias              |   b   | " << p.bias << endl;
      cout << "INPUT VALUES      |   v   | ";
          for(int i=0; i < p.v_inputs.size(); i++)
              cout << p.v_inputs[i] << " ";
          cout << endl;
      cout << "WEIGHTS           |   w   | ";
          for(int i=0; i < p.v_weights.size(); i++)
              cout << p.v_weights[i] << " ";
          cout << endl;
      cout << "DESIRED OUTPUT    |   d   | ";
          for(int i=0; i < p.v_desiredoutputs.size(); i++)
              cout << p.v_desiredoutputs[i] << " ";
          cout << endl;
      cout << endl;
      cout << "OUTPUT                    | " << fire() << endl;
//          for (int i=0; i < p.n_outputs; i++)
//              cout << p.v_outputs[i] << " ";
//          cout << endl;
      cout << "Activation Value          | " << act_val() << endl << endl;
      cout << "------------------------------------COMMANDS-----------------------------------" << endl;
      cout << " t : Train     g : Test" << endl;
      cout << " q : Quit" << endl;
      cout << endl << endl;
}

void edit_input(void)
{
      int x;
      for (int i = 0; i < p.v_inputs.size(); i++)
          {cin >> x; p.v_inputs[i]= x;}
}

void edit_desired_output(void)
{
      int x;
      for (int i = 0; i < p.v_outputs.size(); i++)
          {cin >> x; p.v_outputs[i] = x;}
}

void edit_weights(void)
{
      int x;
      for (int i = 0; i < p.v_weights.size(); i++)
          {cin >> x; p.v_weights[i] = x;}
}

void ptrain(void)
{
      cout << "-in-train-" << endl;
      bool y;
      for (int i = 0; i < p.iterations; i++)
      {
          cout << "--in-train's iteration-" << endl;
          y = fire();
          for (int j = 0; j < p.n_inputs; j++){
              cout << "---in-train's other iteration-" << endl;
              if (y != p.v_desiredoutputs[0])
                  p.v_weights[j+1] = p.v_weights[j] + p.learningrate * ( y - p.v_desiredoutputs[0]);
         }
      }
}

void ptest(void)
{

}

float act_val(void)
{
      float activation_value;
      for (int i = 0; i < p.v_inputs.size(); i++)
          activation_value += p.v_inputs[i] * p.v_weights[i];
      return (activation_value + p.bias);
}

bool fire (void)
{
   return ( (act_val() > 0.0) );
}
